# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import (
    non_inventory_item_list_params,
    non_inventory_item_create_params,
    non_inventory_item_update_params,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.non_inventory_item import NonInventoryItem

__all__ = ["NonInventoryItemsResource", "AsyncNonInventoryItemsResource"]


class NonInventoryItemsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NonInventoryItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return NonInventoryItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NonInventoryItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return NonInventoryItemsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        barcode: non_inventory_item_create_params.Barcode | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        sales_and_purchase_details: non_inventory_item_create_params.SalesAndPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_or_purchase_details: non_inventory_item_create_params.SalesOrPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        sku: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NonInventoryItem:
        """
        Creates a new non-inventory item.

        Args:
          name: The case-insensitive name of this non-inventory item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two non-inventory items could both have the
              `name` "Printer Ink Cartridge", but they could have unique `fullName` values,
              such as "Office Supplies:Printer Ink Cartridge" and "Miscellaneous:Printer Ink
              Cartridge".

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The non-inventory item's barcode.

          class_id: The non-inventory item's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          is_active: Indicates whether this non-inventory item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          parent_id: The parent non-inventory item one level above this one in the hierarchy. For
              example, if this non-inventory item has a `fullName` of "Office Supplies:Printer
              Ink Cartridge", its parent has a `fullName` of "Office Supplies". If this
              non-inventory item is at the top level, this field will be `null`.

          sales_and_purchase_details: Details for non-inventory items that are both purchased and sold, such as
              reimbursable expenses or inventory items that are bought from vendors and sold
              to customers.

              **IMPORTANT**: You must specify either `salesAndPurchaseDetails` or
              `salesOrPurchaseDetails` when creating a non-inventory item, but never both
              because an item cannot have both configurations.

          sales_or_purchase_details: Details for non-inventory items that are exclusively sold or exclusively
              purchased, but not both. This typically applies to non-inventory items (like a
              purchased office supply that isn't resold) or service items (like consulting
              services that are sold but not purchased).

              **IMPORTANT**: You must specify either `salesOrPurchaseDetails` or
              `salesAndPurchaseDetails` when creating a non-inventory item, but never both
              because an item cannot have both configurations.

          sales_tax_code_id: The default sales-tax code for this non-inventory item, determining whether it
              is taxable or non-taxable. This can be overridden at the transaction-line level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          sku: The non-inventory item's stock keeping unit (SKU), which is sometimes the
              manufacturer's part number.

          unit_of_measure_set_id: The unit-of-measure set associated with this non-inventory item, which consists
              of a base unit and related units.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/non-inventory-items",
            body=maybe_transform(
                {
                    "name": name,
                    "barcode": barcode,
                    "class_id": class_id,
                    "external_id": external_id,
                    "is_active": is_active,
                    "parent_id": parent_id,
                    "sales_and_purchase_details": sales_and_purchase_details,
                    "sales_or_purchase_details": sales_or_purchase_details,
                    "sales_tax_code_id": sales_tax_code_id,
                    "sku": sku,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                },
                non_inventory_item_create_params.NonInventoryItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NonInventoryItem,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NonInventoryItem:
        """
        Retrieves a non-inventory item by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the non-inventory item to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/non-inventory-items/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NonInventoryItem,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        barcode: non_inventory_item_update_params.Barcode | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        force_unit_of_measure_change: bool | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        sales_and_purchase_details: non_inventory_item_update_params.SalesAndPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_or_purchase_details: non_inventory_item_update_params.SalesOrPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        sku: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NonInventoryItem:
        """
        Updates an existing non-inventory item.

        Args:
          id: The QuickBooks-assigned unique identifier of the non-inventory item to update.

          revision_number: The current QuickBooks-assigned revision number of the non-inventory item object
              you are updating, which you can get by fetching the object first. Provide the
              most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The non-inventory item's barcode.

          class_id: The non-inventory item's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          force_unit_of_measure_change: Indicates whether to allow changing the non-inventory item's unit-of-measure set
              (using the `unitOfMeasureSetId` field) when the base unit of the new
              unit-of-measure set does not match that of the currently assigned set. Without
              setting this field to `true` in this scenario, the request will fail with an
              error; hence, this field is equivalent to accepting the warning prompt in the
              QuickBooks UI.

              NOTE: Changing the base unit requires you to update the item's
              quantities-on-hand and cost to reflect the new unit; otherwise, these values
              will be inaccurate. Alternatively, consider creating a new item with the desired
              unit-of-measure set and deactivating the old item.

          is_active: Indicates whether this non-inventory item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          name: The case-insensitive name of this non-inventory item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two non-inventory items could both have the
              `name` "Printer Ink Cartridge", but they could have unique `fullName` values,
              such as "Office Supplies:Printer Ink Cartridge" and "Miscellaneous:Printer Ink
              Cartridge".

              Maximum length: 31 characters.

          parent_id: The parent non-inventory item one level above this one in the hierarchy. For
              example, if this non-inventory item has a `fullName` of "Office Supplies:Printer
              Ink Cartridge", its parent has a `fullName` of "Office Supplies". If this
              non-inventory item is at the top level, this field will be `null`.

          sales_and_purchase_details: Details for non-inventory items that are both purchased and sold, such as
              reimbursable expenses or inventory items that are bought from vendors and sold
              to customers.

              **IMPORTANT**: You cannot specify both `salesAndPurchaseDetails` and
              `salesOrPurchaseDetails` when modifying a non-inventory item because an item
              cannot have both configurations.

          sales_or_purchase_details: Details for non-inventory items that are exclusively sold or exclusively
              purchased, but not both. This typically applies to non-inventory items (like a
              purchased office supply that isn't resold) or service items (like consulting
              services that are sold but not purchased).

              **IMPORTANT**: You cannot specify both `salesOrPurchaseDetails` and
              `salesAndPurchaseDetails` when modifying a non-inventory item because an item
              cannot have both configurations.

          sales_tax_code_id: The default sales-tax code for this non-inventory item, determining whether it
              is taxable or non-taxable. This can be overridden at the transaction-line level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          sku: The non-inventory item's stock keeping unit (SKU), which is sometimes the
              manufacturer's part number.

          unit_of_measure_set_id: The unit-of-measure set associated with this non-inventory item, which consists
              of a base unit and related units.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/non-inventory-items/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "barcode": barcode,
                    "class_id": class_id,
                    "force_unit_of_measure_change": force_unit_of_measure_change,
                    "is_active": is_active,
                    "name": name,
                    "parent_id": parent_id,
                    "sales_and_purchase_details": sales_and_purchase_details,
                    "sales_or_purchase_details": sales_or_purchase_details,
                    "sales_tax_code_id": sales_tax_code_id,
                    "sku": sku,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                },
                non_inventory_item_update_params.NonInventoryItemUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NonInventoryItem,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        class_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        full_names: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[NonInventoryItem]:
        """Returns a list of non-inventory items.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_ids: Filter for non-inventory items of these classes. A class is a way end-users can
              categorize non-inventory items in QuickBooks.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          full_names: Filter for specific non-inventory items by their full-name(s), case-insensitive.
              Like `id`, `fullName` is a unique identifier for a non-inventory item, formed by
              by combining the names of its parent objects with its own `name`, separated by
              colons. For example, if a non-inventory item is under "Office Supplies" and has
              the `name` "Printer Ink Cartridge", its `fullName` would be "Office
              Supplies:Printer Ink Cartridge".

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ids: Filter for specific non-inventory items by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for non-inventory items whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for non-inventory items whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for non-inventory items whose `name` is alphabetically greater than or
              equal to this value.

          name_starts_with: Filter for non-inventory items whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for non-inventory items whose `name` is alphabetically less than or equal
              to this value.

          status: Filter for non-inventory items that are active, inactive, or both.

          updated_after: Filter for non-inventory items updated on or after this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 00:00:00 of that day.

          updated_before: Filter for non-inventory items updated on or before this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/non-inventory-items",
            page=SyncCursorPage[NonInventoryItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "class_ids": class_ids,
                        "cursor": cursor,
                        "full_names": full_names,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    non_inventory_item_list_params.NonInventoryItemListParams,
                ),
            ),
            model=NonInventoryItem,
        )


class AsyncNonInventoryItemsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNonInventoryItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNonInventoryItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNonInventoryItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncNonInventoryItemsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        barcode: non_inventory_item_create_params.Barcode | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        sales_and_purchase_details: non_inventory_item_create_params.SalesAndPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_or_purchase_details: non_inventory_item_create_params.SalesOrPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        sku: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NonInventoryItem:
        """
        Creates a new non-inventory item.

        Args:
          name: The case-insensitive name of this non-inventory item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two non-inventory items could both have the
              `name` "Printer Ink Cartridge", but they could have unique `fullName` values,
              such as "Office Supplies:Printer Ink Cartridge" and "Miscellaneous:Printer Ink
              Cartridge".

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The non-inventory item's barcode.

          class_id: The non-inventory item's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          is_active: Indicates whether this non-inventory item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          parent_id: The parent non-inventory item one level above this one in the hierarchy. For
              example, if this non-inventory item has a `fullName` of "Office Supplies:Printer
              Ink Cartridge", its parent has a `fullName` of "Office Supplies". If this
              non-inventory item is at the top level, this field will be `null`.

          sales_and_purchase_details: Details for non-inventory items that are both purchased and sold, such as
              reimbursable expenses or inventory items that are bought from vendors and sold
              to customers.

              **IMPORTANT**: You must specify either `salesAndPurchaseDetails` or
              `salesOrPurchaseDetails` when creating a non-inventory item, but never both
              because an item cannot have both configurations.

          sales_or_purchase_details: Details for non-inventory items that are exclusively sold or exclusively
              purchased, but not both. This typically applies to non-inventory items (like a
              purchased office supply that isn't resold) or service items (like consulting
              services that are sold but not purchased).

              **IMPORTANT**: You must specify either `salesOrPurchaseDetails` or
              `salesAndPurchaseDetails` when creating a non-inventory item, but never both
              because an item cannot have both configurations.

          sales_tax_code_id: The default sales-tax code for this non-inventory item, determining whether it
              is taxable or non-taxable. This can be overridden at the transaction-line level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          sku: The non-inventory item's stock keeping unit (SKU), which is sometimes the
              manufacturer's part number.

          unit_of_measure_set_id: The unit-of-measure set associated with this non-inventory item, which consists
              of a base unit and related units.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/non-inventory-items",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "barcode": barcode,
                    "class_id": class_id,
                    "external_id": external_id,
                    "is_active": is_active,
                    "parent_id": parent_id,
                    "sales_and_purchase_details": sales_and_purchase_details,
                    "sales_or_purchase_details": sales_or_purchase_details,
                    "sales_tax_code_id": sales_tax_code_id,
                    "sku": sku,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                },
                non_inventory_item_create_params.NonInventoryItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NonInventoryItem,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NonInventoryItem:
        """
        Retrieves a non-inventory item by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the non-inventory item to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/non-inventory-items/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NonInventoryItem,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        barcode: non_inventory_item_update_params.Barcode | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        force_unit_of_measure_change: bool | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        sales_and_purchase_details: non_inventory_item_update_params.SalesAndPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_or_purchase_details: non_inventory_item_update_params.SalesOrPurchaseDetails | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        sku: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NonInventoryItem:
        """
        Updates an existing non-inventory item.

        Args:
          id: The QuickBooks-assigned unique identifier of the non-inventory item to update.

          revision_number: The current QuickBooks-assigned revision number of the non-inventory item object
              you are updating, which you can get by fetching the object first. Provide the
              most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The non-inventory item's barcode.

          class_id: The non-inventory item's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          force_unit_of_measure_change: Indicates whether to allow changing the non-inventory item's unit-of-measure set
              (using the `unitOfMeasureSetId` field) when the base unit of the new
              unit-of-measure set does not match that of the currently assigned set. Without
              setting this field to `true` in this scenario, the request will fail with an
              error; hence, this field is equivalent to accepting the warning prompt in the
              QuickBooks UI.

              NOTE: Changing the base unit requires you to update the item's
              quantities-on-hand and cost to reflect the new unit; otherwise, these values
              will be inaccurate. Alternatively, consider creating a new item with the desired
              unit-of-measure set and deactivating the old item.

          is_active: Indicates whether this non-inventory item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          name: The case-insensitive name of this non-inventory item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two non-inventory items could both have the
              `name` "Printer Ink Cartridge", but they could have unique `fullName` values,
              such as "Office Supplies:Printer Ink Cartridge" and "Miscellaneous:Printer Ink
              Cartridge".

              Maximum length: 31 characters.

          parent_id: The parent non-inventory item one level above this one in the hierarchy. For
              example, if this non-inventory item has a `fullName` of "Office Supplies:Printer
              Ink Cartridge", its parent has a `fullName` of "Office Supplies". If this
              non-inventory item is at the top level, this field will be `null`.

          sales_and_purchase_details: Details for non-inventory items that are both purchased and sold, such as
              reimbursable expenses or inventory items that are bought from vendors and sold
              to customers.

              **IMPORTANT**: You cannot specify both `salesAndPurchaseDetails` and
              `salesOrPurchaseDetails` when modifying a non-inventory item because an item
              cannot have both configurations.

          sales_or_purchase_details: Details for non-inventory items that are exclusively sold or exclusively
              purchased, but not both. This typically applies to non-inventory items (like a
              purchased office supply that isn't resold) or service items (like consulting
              services that are sold but not purchased).

              **IMPORTANT**: You cannot specify both `salesOrPurchaseDetails` and
              `salesAndPurchaseDetails` when modifying a non-inventory item because an item
              cannot have both configurations.

          sales_tax_code_id: The default sales-tax code for this non-inventory item, determining whether it
              is taxable or non-taxable. This can be overridden at the transaction-line level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          sku: The non-inventory item's stock keeping unit (SKU), which is sometimes the
              manufacturer's part number.

          unit_of_measure_set_id: The unit-of-measure set associated with this non-inventory item, which consists
              of a base unit and related units.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/non-inventory-items/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "barcode": barcode,
                    "class_id": class_id,
                    "force_unit_of_measure_change": force_unit_of_measure_change,
                    "is_active": is_active,
                    "name": name,
                    "parent_id": parent_id,
                    "sales_and_purchase_details": sales_and_purchase_details,
                    "sales_or_purchase_details": sales_or_purchase_details,
                    "sales_tax_code_id": sales_tax_code_id,
                    "sku": sku,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                },
                non_inventory_item_update_params.NonInventoryItemUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NonInventoryItem,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        class_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        full_names: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[NonInventoryItem, AsyncCursorPage[NonInventoryItem]]:
        """Returns a list of non-inventory items.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_ids: Filter for non-inventory items of these classes. A class is a way end-users can
              categorize non-inventory items in QuickBooks.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          full_names: Filter for specific non-inventory items by their full-name(s), case-insensitive.
              Like `id`, `fullName` is a unique identifier for a non-inventory item, formed by
              by combining the names of its parent objects with its own `name`, separated by
              colons. For example, if a non-inventory item is under "Office Supplies" and has
              the `name` "Printer Ink Cartridge", its `fullName` would be "Office
              Supplies:Printer Ink Cartridge".

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ids: Filter for specific non-inventory items by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for non-inventory items whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for non-inventory items whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for non-inventory items whose `name` is alphabetically greater than or
              equal to this value.

          name_starts_with: Filter for non-inventory items whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for non-inventory items whose `name` is alphabetically less than or equal
              to this value.

          status: Filter for non-inventory items that are active, inactive, or both.

          updated_after: Filter for non-inventory items updated on or after this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 00:00:00 of that day.

          updated_before: Filter for non-inventory items updated on or before this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/non-inventory-items",
            page=AsyncCursorPage[NonInventoryItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "class_ids": class_ids,
                        "cursor": cursor,
                        "full_names": full_names,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    non_inventory_item_list_params.NonInventoryItemListParams,
                ),
            ),
            model=NonInventoryItem,
        )


class NonInventoryItemsResourceWithRawResponse:
    def __init__(self, non_inventory_items: NonInventoryItemsResource) -> None:
        self._non_inventory_items = non_inventory_items

        self.create = to_raw_response_wrapper(
            non_inventory_items.create,
        )
        self.retrieve = to_raw_response_wrapper(
            non_inventory_items.retrieve,
        )
        self.update = to_raw_response_wrapper(
            non_inventory_items.update,
        )
        self.list = to_raw_response_wrapper(
            non_inventory_items.list,
        )


class AsyncNonInventoryItemsResourceWithRawResponse:
    def __init__(self, non_inventory_items: AsyncNonInventoryItemsResource) -> None:
        self._non_inventory_items = non_inventory_items

        self.create = async_to_raw_response_wrapper(
            non_inventory_items.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            non_inventory_items.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            non_inventory_items.update,
        )
        self.list = async_to_raw_response_wrapper(
            non_inventory_items.list,
        )


class NonInventoryItemsResourceWithStreamingResponse:
    def __init__(self, non_inventory_items: NonInventoryItemsResource) -> None:
        self._non_inventory_items = non_inventory_items

        self.create = to_streamed_response_wrapper(
            non_inventory_items.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            non_inventory_items.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            non_inventory_items.update,
        )
        self.list = to_streamed_response_wrapper(
            non_inventory_items.list,
        )


class AsyncNonInventoryItemsResourceWithStreamingResponse:
    def __init__(self, non_inventory_items: AsyncNonInventoryItemsResource) -> None:
        self._non_inventory_items = non_inventory_items

        self.create = async_to_streamed_response_wrapper(
            non_inventory_items.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            non_inventory_items.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            non_inventory_items.update,
        )
        self.list = async_to_streamed_response_wrapper(
            non_inventory_items.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .qbd import (
    QbdResource,
    AsyncQbdResource,
    QbdResourceWithRawResponse,
    AsyncQbdResourceWithRawResponse,
    QbdResourceWithStreamingResponse,
    AsyncQbdResourceWithStreamingResponse,
)
from .bills import (
    BillsResource,
    AsyncBillsResource,
    BillsResourceWithRawResponse,
    AsyncBillsResourceWithRawResponse,
    BillsResourceWithStreamingResponse,
    AsyncBillsResourceWithStreamingResponse,
)
from .checks import (
    ChecksResource,
    AsyncChecksResource,
    ChecksResourceWithRawResponse,
    AsyncChecksResourceWithRawResponse,
    ChecksResourceWithStreamingResponse,
    AsyncChecksResourceWithStreamingResponse,
)
from .classes import (
    ClassesResource,
    AsyncClassesResource,
    ClassesResourceWithRawResponse,
    AsyncClassesResourceWithRawResponse,
    ClassesResourceWithStreamingResponse,
    AsyncClassesResourceWithStreamingResponse,
)
from .company import (
    CompanyResource,
    AsyncCompanyResource,
    CompanyResourceWithRawResponse,
    AsyncCompanyResourceWithRawResponse,
    CompanyResourceWithStreamingResponse,
    AsyncCompanyResourceWithStreamingResponse,
)
from .vendors import (
    VendorsResource,
    AsyncVendorsResource,
    VendorsResourceWithRawResponse,
    AsyncVendorsResourceWithRawResponse,
    VendorsResourceWithStreamingResponse,
    AsyncVendorsResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)
from .employees import (
    EmployeesResource,
    AsyncEmployeesResource,
    EmployeesResourceWithRawResponse,
    AsyncEmployeesResourceWithRawResponse,
    EmployeesResourceWithStreamingResponse,
    AsyncEmployeesResourceWithStreamingResponse,
)
from .estimates import (
    EstimatesResource,
    AsyncEstimatesResource,
    EstimatesResourceWithRawResponse,
    AsyncEstimatesResourceWithRawResponse,
    EstimatesResourceWithStreamingResponse,
    AsyncEstimatesResourceWithStreamingResponse,
)
from .transfers import (
    TransfersResource,
    AsyncTransfersResource,
    TransfersResourceWithRawResponse,
    AsyncTransfersResourceWithRawResponse,
    TransfersResourceWithStreamingResponse,
    AsyncTransfersResourceWithStreamingResponse,
)
from .preferences import (
    PreferencesResource,
    AsyncPreferencesResource,
    PreferencesResourceWithRawResponse,
    AsyncPreferencesResourceWithRawResponse,
    PreferencesResourceWithStreamingResponse,
    AsyncPreferencesResourceWithStreamingResponse,
)
from .credit_memos import (
    CreditMemosResource,
    AsyncCreditMemosResource,
    CreditMemosResourceWithRawResponse,
    AsyncCreditMemosResourceWithRawResponse,
    CreditMemosResourceWithStreamingResponse,
    AsyncCreditMemosResourceWithStreamingResponse,
)
from .price_levels import (
    PriceLevelsResource,
    AsyncPriceLevelsResource,
    PriceLevelsResourceWithRawResponse,
    AsyncPriceLevelsResourceWithRawResponse,
    PriceLevelsResourceWithStreamingResponse,
    AsyncPriceLevelsResourceWithStreamingResponse,
)
from .sales_orders import (
    SalesOrdersResource,
    AsyncSalesOrdersResource,
    SalesOrdersResourceWithRawResponse,
    AsyncSalesOrdersResourceWithRawResponse,
    SalesOrdersResourceWithStreamingResponse,
    AsyncSalesOrdersResourceWithStreamingResponse,
)
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)
from .item_receipts import (
    ItemReceiptsResource,
    AsyncItemReceiptsResource,
    ItemReceiptsResourceWithRawResponse,
    AsyncItemReceiptsResourceWithRawResponse,
    ItemReceiptsResourceWithStreamingResponse,
    AsyncItemReceiptsResourceWithStreamingResponse,
)
from .service_items import (
    ServiceItemsResource,
    AsyncServiceItemsResource,
    ServiceItemsResourceWithRawResponse,
    AsyncServiceItemsResourceWithRawResponse,
    ServiceItemsResourceWithStreamingResponse,
    AsyncServiceItemsResourceWithStreamingResponse,
)
from .discount_items import (
    DiscountItemsResource,
    AsyncDiscountItemsResource,
    DiscountItemsResourceWithRawResponse,
    AsyncDiscountItemsResourceWithRawResponse,
    DiscountItemsResourceWithStreamingResponse,
    AsyncDiscountItemsResourceWithStreamingResponse,
)
from .sales_receipts import (
    SalesReceiptsResource,
    AsyncSalesReceiptsResource,
    SalesReceiptsResourceWithRawResponse,
    AsyncSalesReceiptsResourceWithRawResponse,
    SalesReceiptsResourceWithStreamingResponse,
    AsyncSalesReceiptsResourceWithStreamingResponse,
)
from .standard_terms import (
    StandardTermsResource,
    AsyncStandardTermsResource,
    StandardTermsResourceWithRawResponse,
    AsyncStandardTermsResourceWithRawResponse,
    StandardTermsResourceWithStreamingResponse,
    AsyncStandardTermsResourceWithStreamingResponse,
)
from .subtotal_items import (
    SubtotalItemsResource,
    AsyncSubtotalItemsResource,
    SubtotalItemsResourceWithRawResponse,
    AsyncSubtotalItemsResourceWithRawResponse,
    SubtotalItemsResourceWithStreamingResponse,
    AsyncSubtotalItemsResourceWithStreamingResponse,
)
from .vendor_credits import (
    VendorCreditsResource,
    AsyncVendorCreditsResource,
    VendorCreditsResourceWithRawResponse,
    AsyncVendorCreditsResourceWithRawResponse,
    VendorCreditsResourceWithStreamingResponse,
    AsyncVendorCreditsResourceWithStreamingResponse,
)
from .inventory_items import (
    InventoryItemsResource,
    AsyncInventoryItemsResource,
    InventoryItemsResourceWithRawResponse,
    AsyncInventoryItemsResourceWithRawResponse,
    InventoryItemsResourceWithStreamingResponse,
    AsyncInventoryItemsResourceWithStreamingResponse,
)
from .inventory_sites import (
    InventorySitesResource,
    AsyncInventorySitesResource,
    InventorySitesResourceWithRawResponse,
    AsyncInventorySitesResourceWithRawResponse,
    InventorySitesResourceWithStreamingResponse,
    AsyncInventorySitesResourceWithStreamingResponse,
)
from .journal_entries import (
    JournalEntriesResource,
    AsyncJournalEntriesResource,
    JournalEntriesResourceWithRawResponse,
    AsyncJournalEntriesResourceWithRawResponse,
    JournalEntriesResourceWithStreamingResponse,
    AsyncJournalEntriesResourceWithStreamingResponse,
)
from .purchase_orders import (
    PurchaseOrdersResource,
    AsyncPurchaseOrdersResource,
    PurchaseOrdersResourceWithRawResponse,
    AsyncPurchaseOrdersResourceWithRawResponse,
    PurchaseOrdersResourceWithStreamingResponse,
    AsyncPurchaseOrdersResourceWithStreamingResponse,
)
from .sales_tax_codes import (
    SalesTaxCodesResource,
    AsyncSalesTaxCodesResource,
    SalesTaxCodesResourceWithRawResponse,
    AsyncSalesTaxCodesResourceWithRawResponse,
    SalesTaxCodesResourceWithStreamingResponse,
    AsyncSalesTaxCodesResourceWithStreamingResponse,
)
from .sales_tax_items import (
    SalesTaxItemsResource,
    AsyncSalesTaxItemsResource,
    SalesTaxItemsResourceWithRawResponse,
    AsyncSalesTaxItemsResourceWithRawResponse,
    SalesTaxItemsResourceWithStreamingResponse,
    AsyncSalesTaxItemsResourceWithStreamingResponse,
)
from .receive_payments import (
    ReceivePaymentsResource,
    AsyncReceivePaymentsResource,
    ReceivePaymentsResourceWithRawResponse,
    AsyncReceivePaymentsResourceWithRawResponse,
    ReceivePaymentsResourceWithStreamingResponse,
    AsyncReceivePaymentsResourceWithStreamingResponse,
)
from .account_tax_lines import (
    AccountTaxLinesResource,
    AsyncAccountTaxLinesResource,
    AccountTaxLinesResourceWithRawResponse,
    AsyncAccountTaxLinesResourceWithRawResponse,
    AccountTaxLinesResourceWithStreamingResponse,
    AsyncAccountTaxLinesResourceWithStreamingResponse,
)
from .date_driven_terms import (
    DateDrivenTermsResource,
    AsyncDateDrivenTermsResource,
    DateDrivenTermsResourceWithRawResponse,
    AsyncDateDrivenTermsResourceWithRawResponse,
    DateDrivenTermsResourceWithStreamingResponse,
    AsyncDateDrivenTermsResourceWithStreamingResponse,
)
from .payroll_wage_items import (
    PayrollWageItemsResource,
    AsyncPayrollWageItemsResource,
    PayrollWageItemsResourceWithRawResponse,
    AsyncPayrollWageItemsResourceWithRawResponse,
    PayrollWageItemsResourceWithStreamingResponse,
    AsyncPayrollWageItemsResourceWithStreamingResponse,
)
from .bill_check_payments import (
    BillCheckPaymentsResource,
    AsyncBillCheckPaymentsResource,
    BillCheckPaymentsResourceWithRawResponse,
    AsyncBillCheckPaymentsResourceWithRawResponse,
    BillCheckPaymentsResourceWithStreamingResponse,
    AsyncBillCheckPaymentsResourceWithStreamingResponse,
)
from .credit_card_charges import (
    CreditCardChargesResource,
    AsyncCreditCardChargesResource,
    CreditCardChargesResourceWithRawResponse,
    AsyncCreditCardChargesResourceWithRawResponse,
    CreditCardChargesResourceWithStreamingResponse,
    AsyncCreditCardChargesResourceWithStreamingResponse,
)
from .credit_card_credits import (
    CreditCardCreditsResource,
    AsyncCreditCardCreditsResource,
    CreditCardCreditsResourceWithRawResponse,
    AsyncCreditCardCreditsResourceWithRawResponse,
    CreditCardCreditsResourceWithStreamingResponse,
    AsyncCreditCardCreditsResourceWithStreamingResponse,
)
from .non_inventory_items import (
    NonInventoryItemsResource,
    AsyncNonInventoryItemsResource,
    NonInventoryItemsResourceWithRawResponse,
    AsyncNonInventoryItemsResourceWithRawResponse,
    NonInventoryItemsResourceWithStreamingResponse,
    AsyncNonInventoryItemsResourceWithStreamingResponse,
)
from .inventory_adjustments import (
    InventoryAdjustmentsResource,
    AsyncInventoryAdjustmentsResource,
    InventoryAdjustmentsResourceWithRawResponse,
    AsyncInventoryAdjustmentsResourceWithRawResponse,
    InventoryAdjustmentsResourceWithStreamingResponse,
    AsyncInventoryAdjustmentsResourceWithStreamingResponse,
)
from .sales_representatives import (
    SalesRepresentativesResource,
    AsyncSalesRepresentativesResource,
    SalesRepresentativesResourceWithRawResponse,
    AsyncSalesRepresentativesResourceWithRawResponse,
    SalesRepresentativesResourceWithStreamingResponse,
    AsyncSalesRepresentativesResourceWithStreamingResponse,
)
from .inventory_assembly_items import (
    InventoryAssemblyItemsResource,
    AsyncInventoryAssemblyItemsResource,
    InventoryAssemblyItemsResourceWithRawResponse,
    AsyncInventoryAssemblyItemsResourceWithRawResponse,
    InventoryAssemblyItemsResourceWithStreamingResponse,
    AsyncInventoryAssemblyItemsResourceWithStreamingResponse,
)
from .time_tracking_activities import (
    TimeTrackingActivitiesResource,
    AsyncTimeTrackingActivitiesResource,
    TimeTrackingActivitiesResourceWithRawResponse,
    AsyncTimeTrackingActivitiesResourceWithRawResponse,
    TimeTrackingActivitiesResourceWithStreamingResponse,
    AsyncTimeTrackingActivitiesResourceWithStreamingResponse,
)
from .bill_credit_card_payments import (
    BillCreditCardPaymentsResource,
    AsyncBillCreditCardPaymentsResource,
    BillCreditCardPaymentsResourceWithRawResponse,
    AsyncBillCreditCardPaymentsResourceWithRawResponse,
    BillCreditCardPaymentsResourceWithStreamingResponse,
    AsyncBillCreditCardPaymentsResourceWithStreamingResponse,
)

__all__ = [
    "AccountTaxLinesResource",
    "AsyncAccountTaxLinesResource",
    "AccountTaxLinesResourceWithRawResponse",
    "AsyncAccountTaxLinesResourceWithRawResponse",
    "AccountTaxLinesResourceWithStreamingResponse",
    "AsyncAccountTaxLinesResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "BillCheckPaymentsResource",
    "AsyncBillCheckPaymentsResource",
    "BillCheckPaymentsResourceWithRawResponse",
    "AsyncBillCheckPaymentsResourceWithRawResponse",
    "BillCheckPaymentsResourceWithStreamingResponse",
    "AsyncBillCheckPaymentsResourceWithStreamingResponse",
    "BillCreditCardPaymentsResource",
    "AsyncBillCreditCardPaymentsResource",
    "BillCreditCardPaymentsResourceWithRawResponse",
    "AsyncBillCreditCardPaymentsResourceWithRawResponse",
    "BillCreditCardPaymentsResourceWithStreamingResponse",
    "AsyncBillCreditCardPaymentsResourceWithStreamingResponse",
    "BillsResource",
    "AsyncBillsResource",
    "BillsResourceWithRawResponse",
    "AsyncBillsResourceWithRawResponse",
    "BillsResourceWithStreamingResponse",
    "AsyncBillsResourceWithStreamingResponse",
    "ChecksResource",
    "AsyncChecksResource",
    "ChecksResourceWithRawResponse",
    "AsyncChecksResourceWithRawResponse",
    "ChecksResourceWithStreamingResponse",
    "AsyncChecksResourceWithStreamingResponse",
    "ClassesResource",
    "AsyncClassesResource",
    "ClassesResourceWithRawResponse",
    "AsyncClassesResourceWithRawResponse",
    "ClassesResourceWithStreamingResponse",
    "AsyncClassesResourceWithStreamingResponse",
    "CompanyResource",
    "AsyncCompanyResource",
    "CompanyResourceWithRawResponse",
    "AsyncCompanyResourceWithRawResponse",
    "CompanyResourceWithStreamingResponse",
    "AsyncCompanyResourceWithStreamingResponse",
    "CreditCardChargesResource",
    "AsyncCreditCardChargesResource",
    "CreditCardChargesResourceWithRawResponse",
    "AsyncCreditCardChargesResourceWithRawResponse",
    "CreditCardChargesResourceWithStreamingResponse",
    "AsyncCreditCardChargesResourceWithStreamingResponse",
    "CreditCardCreditsResource",
    "AsyncCreditCardCreditsResource",
    "CreditCardCreditsResourceWithRawResponse",
    "AsyncCreditCardCreditsResourceWithRawResponse",
    "CreditCardCreditsResourceWithStreamingResponse",
    "AsyncCreditCardCreditsResourceWithStreamingResponse",
    "CreditMemosResource",
    "AsyncCreditMemosResource",
    "CreditMemosResourceWithRawResponse",
    "AsyncCreditMemosResourceWithRawResponse",
    "CreditMemosResourceWithStreamingResponse",
    "AsyncCreditMemosResourceWithStreamingResponse",
    "CustomersResource",
    "AsyncCustomersResource",
    "CustomersResourceWithRawResponse",
    "AsyncCustomersResourceWithRawResponse",
    "CustomersResourceWithStreamingResponse",
    "AsyncCustomersResourceWithStreamingResponse",
    "DateDrivenTermsResource",
    "AsyncDateDrivenTermsResource",
    "DateDrivenTermsResourceWithRawResponse",
    "AsyncDateDrivenTermsResourceWithRawResponse",
    "DateDrivenTermsResourceWithStreamingResponse",
    "AsyncDateDrivenTermsResourceWithStreamingResponse",
    "DiscountItemsResource",
    "AsyncDiscountItemsResource",
    "DiscountItemsResourceWithRawResponse",
    "AsyncDiscountItemsResourceWithRawResponse",
    "DiscountItemsResourceWithStreamingResponse",
    "AsyncDiscountItemsResourceWithStreamingResponse",
    "EmployeesResource",
    "AsyncEmployeesResource",
    "EmployeesResourceWithRawResponse",
    "AsyncEmployeesResourceWithRawResponse",
    "EmployeesResourceWithStreamingResponse",
    "AsyncEmployeesResourceWithStreamingResponse",
    "EstimatesResource",
    "AsyncEstimatesResource",
    "EstimatesResourceWithRawResponse",
    "AsyncEstimatesResourceWithRawResponse",
    "EstimatesResourceWithStreamingResponse",
    "AsyncEstimatesResourceWithStreamingResponse",
    "InventoryAdjustmentsResource",
    "AsyncInventoryAdjustmentsResource",
    "InventoryAdjustmentsResourceWithRawResponse",
    "AsyncInventoryAdjustmentsResourceWithRawResponse",
    "InventoryAdjustmentsResourceWithStreamingResponse",
    "AsyncInventoryAdjustmentsResourceWithStreamingResponse",
    "InventoryAssemblyItemsResource",
    "AsyncInventoryAssemblyItemsResource",
    "InventoryAssemblyItemsResourceWithRawResponse",
    "AsyncInventoryAssemblyItemsResourceWithRawResponse",
    "InventoryAssemblyItemsResourceWithStreamingResponse",
    "AsyncInventoryAssemblyItemsResourceWithStreamingResponse",
    "InventoryItemsResource",
    "AsyncInventoryItemsResource",
    "InventoryItemsResourceWithRawResponse",
    "AsyncInventoryItemsResourceWithRawResponse",
    "InventoryItemsResourceWithStreamingResponse",
    "AsyncInventoryItemsResourceWithStreamingResponse",
    "InventorySitesResource",
    "AsyncInventorySitesResource",
    "InventorySitesResourceWithRawResponse",
    "AsyncInventorySitesResourceWithRawResponse",
    "InventorySitesResourceWithStreamingResponse",
    "AsyncInventorySitesResourceWithStreamingResponse",
    "InvoicesResource",
    "AsyncInvoicesResource",
    "InvoicesResourceWithRawResponse",
    "AsyncInvoicesResourceWithRawResponse",
    "InvoicesResourceWithStreamingResponse",
    "AsyncInvoicesResourceWithStreamingResponse",
    "ItemReceiptsResource",
    "AsyncItemReceiptsResource",
    "ItemReceiptsResourceWithRawResponse",
    "AsyncItemReceiptsResourceWithRawResponse",
    "ItemReceiptsResourceWithStreamingResponse",
    "AsyncItemReceiptsResourceWithStreamingResponse",
    "JournalEntriesResource",
    "AsyncJournalEntriesResource",
    "JournalEntriesResourceWithRawResponse",
    "AsyncJournalEntriesResourceWithRawResponse",
    "JournalEntriesResourceWithStreamingResponse",
    "AsyncJournalEntriesResourceWithStreamingResponse",
    "NonInventoryItemsResource",
    "AsyncNonInventoryItemsResource",
    "NonInventoryItemsResourceWithRawResponse",
    "AsyncNonInventoryItemsResourceWithRawResponse",
    "NonInventoryItemsResourceWithStreamingResponse",
    "AsyncNonInventoryItemsResourceWithStreamingResponse",
    "PayrollWageItemsResource",
    "AsyncPayrollWageItemsResource",
    "PayrollWageItemsResourceWithRawResponse",
    "AsyncPayrollWageItemsResourceWithRawResponse",
    "PayrollWageItemsResourceWithStreamingResponse",
    "AsyncPayrollWageItemsResourceWithStreamingResponse",
    "PreferencesResource",
    "AsyncPreferencesResource",
    "PreferencesResourceWithRawResponse",
    "AsyncPreferencesResourceWithRawResponse",
    "PreferencesResourceWithStreamingResponse",
    "AsyncPreferencesResourceWithStreamingResponse",
    "PriceLevelsResource",
    "AsyncPriceLevelsResource",
    "PriceLevelsResourceWithRawResponse",
    "AsyncPriceLevelsResourceWithRawResponse",
    "PriceLevelsResourceWithStreamingResponse",
    "AsyncPriceLevelsResourceWithStreamingResponse",
    "PurchaseOrdersResource",
    "AsyncPurchaseOrdersResource",
    "PurchaseOrdersResourceWithRawResponse",
    "AsyncPurchaseOrdersResourceWithRawResponse",
    "PurchaseOrdersResourceWithStreamingResponse",
    "AsyncPurchaseOrdersResourceWithStreamingResponse",
    "ReceivePaymentsResource",
    "AsyncReceivePaymentsResource",
    "ReceivePaymentsResourceWithRawResponse",
    "AsyncReceivePaymentsResourceWithRawResponse",
    "ReceivePaymentsResourceWithStreamingResponse",
    "AsyncReceivePaymentsResourceWithStreamingResponse",
    "SalesOrdersResource",
    "AsyncSalesOrdersResource",
    "SalesOrdersResourceWithRawResponse",
    "AsyncSalesOrdersResourceWithRawResponse",
    "SalesOrdersResourceWithStreamingResponse",
    "AsyncSalesOrdersResourceWithStreamingResponse",
    "SalesReceiptsResource",
    "AsyncSalesReceiptsResource",
    "SalesReceiptsResourceWithRawResponse",
    "AsyncSalesReceiptsResourceWithRawResponse",
    "SalesReceiptsResourceWithStreamingResponse",
    "AsyncSalesReceiptsResourceWithStreamingResponse",
    "SalesRepresentativesResource",
    "AsyncSalesRepresentativesResource",
    "SalesRepresentativesResourceWithRawResponse",
    "AsyncSalesRepresentativesResourceWithRawResponse",
    "SalesRepresentativesResourceWithStreamingResponse",
    "AsyncSalesRepresentativesResourceWithStreamingResponse",
    "SalesTaxCodesResource",
    "AsyncSalesTaxCodesResource",
    "SalesTaxCodesResourceWithRawResponse",
    "AsyncSalesTaxCodesResourceWithRawResponse",
    "SalesTaxCodesResourceWithStreamingResponse",
    "AsyncSalesTaxCodesResourceWithStreamingResponse",
    "SalesTaxItemsResource",
    "AsyncSalesTaxItemsResource",
    "SalesTaxItemsResourceWithRawResponse",
    "AsyncSalesTaxItemsResourceWithRawResponse",
    "SalesTaxItemsResourceWithStreamingResponse",
    "AsyncSalesTaxItemsResourceWithStreamingResponse",
    "ServiceItemsResource",
    "AsyncServiceItemsResource",
    "ServiceItemsResourceWithRawResponse",
    "AsyncServiceItemsResourceWithRawResponse",
    "ServiceItemsResourceWithStreamingResponse",
    "AsyncServiceItemsResourceWithStreamingResponse",
    "StandardTermsResource",
    "AsyncStandardTermsResource",
    "StandardTermsResourceWithRawResponse",
    "AsyncStandardTermsResourceWithRawResponse",
    "StandardTermsResourceWithStreamingResponse",
    "AsyncStandardTermsResourceWithStreamingResponse",
    "SubtotalItemsResource",
    "AsyncSubtotalItemsResource",
    "SubtotalItemsResourceWithRawResponse",
    "AsyncSubtotalItemsResourceWithRawResponse",
    "SubtotalItemsResourceWithStreamingResponse",
    "AsyncSubtotalItemsResourceWithStreamingResponse",
    "TimeTrackingActivitiesResource",
    "AsyncTimeTrackingActivitiesResource",
    "TimeTrackingActivitiesResourceWithRawResponse",
    "AsyncTimeTrackingActivitiesResourceWithRawResponse",
    "TimeTrackingActivitiesResourceWithStreamingResponse",
    "AsyncTimeTrackingActivitiesResourceWithStreamingResponse",
    "TransactionsResource",
    "AsyncTransactionsResource",
    "TransactionsResourceWithRawResponse",
    "AsyncTransactionsResourceWithRawResponse",
    "TransactionsResourceWithStreamingResponse",
    "AsyncTransactionsResourceWithStreamingResponse",
    "TransfersResource",
    "AsyncTransfersResource",
    "TransfersResourceWithRawResponse",
    "AsyncTransfersResourceWithRawResponse",
    "TransfersResourceWithStreamingResponse",
    "AsyncTransfersResourceWithStreamingResponse",
    "VendorCreditsResource",
    "AsyncVendorCreditsResource",
    "VendorCreditsResourceWithRawResponse",
    "AsyncVendorCreditsResourceWithRawResponse",
    "VendorCreditsResourceWithStreamingResponse",
    "AsyncVendorCreditsResourceWithStreamingResponse",
    "VendorsResource",
    "AsyncVendorsResource",
    "VendorsResourceWithRawResponse",
    "AsyncVendorsResourceWithRawResponse",
    "VendorsResourceWithStreamingResponse",
    "AsyncVendorsResourceWithStreamingResponse",
    "QbdResource",
    "AsyncQbdResource",
    "QbdResourceWithRawResponse",
    "AsyncQbdResourceWithRawResponse",
    "QbdResourceWithStreamingResponse",
    "AsyncQbdResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ClassListParams"]


class ClassListParams(TypedDict, total=False):
    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    full_names: Annotated[List[str], PropertyInfo(alias="fullNames")]
    """Filter for specific classes by their full-name(s), case-insensitive.

    Like `id`, `fullName` is a unique identifier for a class, formed by by combining
    the names of its parent objects with its own `name`, separated by colons. For
    example, if a class is under "Department" and has the `name` "Marketing", its
    `fullName` would be "Department:Marketing".

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    ids: List[str]
    """Filter for specific classes by their QuickBooks-assigned unique identifier(s).

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    limit: int
    """The maximum number of objects to return.

    **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
    classes. This parameter will limit the response size, but you cannot fetch
    subsequent results using a cursor. For pagination, use the name-range parameters
    instead (e.g., `nameFrom=A&nameTo=B`).

    When this parameter is omitted, the endpoint returns all classes without limit,
    unlike paginated endpoints which default to 150 records. This is acceptable
    because classes typically have low record counts.
    """

    name_contains: Annotated[str, PropertyInfo(alias="nameContains")]
    """Filter for classes whose `name` contains this substring, case-insensitive.

    NOTE: If you use this parameter, you cannot also use `nameStartsWith` or
    `nameEndsWith`.
    """

    name_ends_with: Annotated[str, PropertyInfo(alias="nameEndsWith")]
    """Filter for classes whose `name` ends with this substring, case-insensitive.

    NOTE: If you use this parameter, you cannot also use `nameContains` or
    `nameStartsWith`.
    """

    name_from: Annotated[str, PropertyInfo(alias="nameFrom")]
    """
    Filter for classes whose `name` is alphabetically greater than or equal to this
    value.
    """

    name_starts_with: Annotated[str, PropertyInfo(alias="nameStartsWith")]
    """Filter for classes whose `name` starts with this substring, case-insensitive.

    NOTE: If you use this parameter, you cannot also use `nameContains` or
    `nameEndsWith`.
    """

    name_to: Annotated[str, PropertyInfo(alias="nameTo")]
    """
    Filter for classes whose `name` is alphabetically less than or equal to this
    value.
    """

    status: Literal["active", "all", "inactive"]
    """Filter for classes that are active, inactive, or both."""

    updated_after: Annotated[str, PropertyInfo(alias="updatedAfter")]
    """
    Filter for classes updated on or after this date and time, in ISO 8601 format
    (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
    assumed to be 00:00:00 of that day.
    """

    updated_before: Annotated[str, PropertyInfo(alias="updatedBefore")]
    """
    Filter for classes updated on or before this date and time, in ISO 8601 format
    (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
    assumed to be 23:59:59 of that day.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CreditCardChargeDeleteResponse"]


class CreditCardChargeDeleteResponse(BaseModel):
    id: str
    """The QuickBooks-assigned unique identifier of the deleted credit card charge."""

    deleted: bool
    """Indicates whether the credit card charge was deleted."""

    object_type: Literal["qbd_credit_card_charge"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_credit_card_charge"`."""

    ref_number: Optional[str] = FieldInfo(alias="refNumber", default=None)
    """
    The case-sensitive user-defined reference number of the deleted credit card
    charge.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SalesRepresentativeUpdateParams"]


class SalesRepresentativeUpdateParams(TypedDict, total=False):
    revision_number: Required[Annotated[str, PropertyInfo(alias="revisionNumber")]]
    """
    The current QuickBooks-assigned revision number of the sales representative
    object you are updating, which you can get by fetching the object first. Provide
    the most recent `revisionNumber` to ensure you're working with the latest data;
    otherwise, the update will return an error.
    """

    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    entity_id: Annotated[str, PropertyInfo(alias="entityId")]
    """
    The sales representative's corresponding person entity in QuickBooks, stored as
    either an employee, vendor, or other-name entry.
    """

    initial: str
    """The initials of this sales representative's name."""

    is_active: Annotated[bool, PropertyInfo(alias="isActive")]
    """Indicates whether this sales representative is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import receive_payment_list_params, receive_payment_create_params, receive_payment_update_params
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.receive_payment import ReceivePayment
from ...types.qbd.receive_payment_delete_response import ReceivePaymentDeleteResponse

__all__ = ["ReceivePaymentsResource", "AsyncReceivePaymentsResource"]


class ReceivePaymentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReceivePaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return ReceivePaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReceivePaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return ReceivePaymentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        customer_id: str,
        total_amount: str,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        apply_to_transactions: Iterable[receive_payment_create_params.ApplyToTransaction] | NotGiven = NOT_GIVEN,
        credit_card_transaction: receive_payment_create_params.CreditCardTransaction | NotGiven = NOT_GIVEN,
        deposit_to_account_id: str | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        is_auto_apply: bool | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payment_method_id: str | NotGiven = NOT_GIVEN,
        receivables_account_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePayment:
        """
        Creates a new receive-payment.

        Args:
          customer_id: The customer or customer-job to which the payment for this receive-payment is
              credited.

          total_amount: The total monetary amount of this receive-payment, represented as a decimal
              string.

              **NOTE**: The sum of the `paymentAmount` amounts in the `applyToTransactions`
              array cannot exceed the `totalAmount`, or you will receive an error.

          transaction_date: The date of this receive-payment, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          apply_to_transactions: The invoices to be paid by this receive-payment. This will create a link between
              this receive-payment and the specified invoices.

              **IMPORTANT**: In each `applyToTransactions` object, you must specify either
              `paymentAmount`, `applyCredits`, `discountAmount`, or any combination of these;
              if none of these are specified, you will receive an error for an empty
              transaction.

              **IMPORTANT**: The target invoice must have `isPaid=false`, otherwise,
              QuickBooks will report this object as "cannot be found".

              **NOTE**: You must specify either `isAutoApply` or `applyToTransactions` when
              creating a receive-payment, but never both.

          credit_card_transaction: The credit card transaction data for this receive-payment's payment when using
              QuickBooks Merchant Services (QBMS). If specifying this field, you must also
              specify the `paymentMethod` field.

          deposit_to_account_id: The account where the funds for this receive-payment will be or have been
              deposited. If omitted, QuickBooks will use the default Undeposited Funds
              account.

          exchange_rate: The market exchange rate between this receive-payment's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          is_auto_apply: When `true`, QuickBooks applies `totalAmount` to any outstanding transaction
              that exactly matches `totalAmount`. If no exact match is found, this
              receive-payment is applied to the oldest outstanding transaction for the
              customer-job. When `false`, QuickBooks records the payment but does not apply it
              to any specific transaction, causing the amount to appear as a credit on the
              customer-job's next transaction.

              **IMPORTANT**: You must specify either `isAutoApply` or `applyToTransactions`
              when creating a receive-payment, but never both.

          memo: A memo or note for this receive-payment that will be displayed at the beginning
              of reports containing details about this receive-payment.

          payment_method_id: The receive-payment's payment method (e.g., cash, check, credit card).

              **NOTE**: If this receive-payment contains credit card transaction data supplied
              from QuickBooks Merchant Services (QBMS) transaction responses, you must specify
              a credit card payment method (e.g., "Visa", "MasterCard", etc.).

          receivables_account_id: The Accounts-Receivable (A/R) account to which this receive-payment is assigned,
              used to track the amount owed. If not specified, QuickBooks Desktop will use its
              default A/R account.

              **IMPORTANT**: If this receive-payment is linked to other transactions, this A/R
              account must match the `receivablesAccount` used in all linked transactions. For
              example, when refunding a credit card payment, the A/R account must match the
              one used in the original credit transactions being refunded.

          ref_number: The case-sensitive user-defined reference number for this receive-payment, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.
              When left blank in this create request, this field will be left blank in
              QuickBooks (i.e., it does _not_ auto-increment).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/receive-payments",
            body=maybe_transform(
                {
                    "customer_id": customer_id,
                    "total_amount": total_amount,
                    "transaction_date": transaction_date,
                    "apply_to_transactions": apply_to_transactions,
                    "credit_card_transaction": credit_card_transaction,
                    "deposit_to_account_id": deposit_to_account_id,
                    "exchange_rate": exchange_rate,
                    "external_id": external_id,
                    "is_auto_apply": is_auto_apply,
                    "memo": memo,
                    "payment_method_id": payment_method_id,
                    "receivables_account_id": receivables_account_id,
                    "ref_number": ref_number,
                },
                receive_payment_create_params.ReceivePaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePayment,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePayment:
        """
        Retrieves a receive-payment by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the receive-payment to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/receive-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePayment,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        apply_to_transactions: Iterable[receive_payment_update_params.ApplyToTransaction] | NotGiven = NOT_GIVEN,
        credit_card_transaction: receive_payment_update_params.CreditCardTransaction | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        deposit_to_account_id: str | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payment_method_id: str | NotGiven = NOT_GIVEN,
        receivables_account_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        total_amount: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePayment:
        """
        Updates an existing receive-payment.

        Args:
          id: The QuickBooks-assigned unique identifier of the receive-payment to update.

          revision_number: The current QuickBooks-assigned revision number of the receive-payment object
              you are updating, which you can get by fetching the object first. Provide the
              most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          apply_to_transactions: The invoices to be paid by this receive-payment. This will create a link between
              this receive-payment and the specified invoices.

              **IMPORTANT**: In each `applyToTransactions` object, you must specify either
              `paymentAmount`, `applyCredits`, `discountAmount`, or any combination of these;
              if none of these are specified, you will receive an error for an empty
              transaction.

              **IMPORTANT**: The target invoice must have `isPaid=false`, otherwise,
              QuickBooks will report this object as "cannot be found".

          credit_card_transaction: The credit card transaction data for this receive-payment's payment when using
              QuickBooks Merchant Services (QBMS). If specifying this field, you must also
              specify the `paymentMethod` field.

          customer_id: The customer or customer-job to which the payment for this receive-payment is
              credited.

          deposit_to_account_id: The account where the funds for this receive-payment will be or have been
              deposited.

          exchange_rate: The market exchange rate between this receive-payment's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          memo: A memo or note for this receive-payment that will be displayed at the beginning
              of reports containing details about this receive-payment.

          payment_method_id: The receive-payment's payment method (e.g., cash, check, credit card).

          receivables_account_id: The Accounts-Receivable (A/R) account to which this receive-payment is assigned,
              used to track the amount owed. If not specified, QuickBooks Desktop will use its
              default A/R account.

              **IMPORTANT**: If this receive-payment is linked to other transactions, this A/R
              account must match the `receivablesAccount` used in all linked transactions. For
              example, when refunding a credit card payment, the A/R account must match the
              one used in the original credit transactions being refunded.

          ref_number: The case-sensitive user-defined reference number for this receive-payment, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          total_amount: The total monetary amount of this receive-payment, represented as a decimal
              string.

              **NOTE**: The sum of the `paymentAmount` amounts in the `applyToTransactions`
              array cannot exceed the `totalAmount`, or you will receive an error.

          transaction_date: The date of this receive-payment, in ISO 8601 format (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/receive-payments/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "apply_to_transactions": apply_to_transactions,
                    "credit_card_transaction": credit_card_transaction,
                    "customer_id": customer_id,
                    "deposit_to_account_id": deposit_to_account_id,
                    "exchange_rate": exchange_rate,
                    "memo": memo,
                    "payment_method_id": payment_method_id,
                    "receivables_account_id": receivables_account_id,
                    "ref_number": ref_number,
                    "total_amount": total_amount,
                    "transaction_date": transaction_date,
                },
                receive_payment_update_params.ReceivePaymentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePayment,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        customer_ids: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[ReceivePayment]:
        """Returns a list of receive-payments.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for receive-payments associated with these accounts.

          currency_ids: Filter for receive-payments in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          customer_ids: Filter for receive-payments received from these customers.

          ids: Filter for specific receive-payments by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          ref_number_contains: Filter for receive-payments whose `refNumber` contains this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with: Filter for receive-payments whose `refNumber` ends with this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for receive-payments whose `refNumber` is greater than or equal to this
              value. If omitted, the range will begin with the first number of the list. Uses
              a numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          ref_numbers: Filter for specific receive-payments by their ref-number(s), case-sensitive. In
              QuickBooks, ref-numbers are not required to be unique and can be arbitrarily
              changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ref_number_starts_with:
              Filter for receive-payments whose `refNumber` starts with this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberEndsWith`.

          ref_number_to: Filter for receive-payments whose `refNumber` is less than or equal to this
              value. If omitted, the range will end with the last number of the list. Uses a
              numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          transaction_date_from: Filter for receive-payments whose `date` field is on or after this date, in ISO
              8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for receive-payments whose `date` field is on or before this date, in ISO
              8601 format (YYYY-MM-DD).

          updated_after: Filter for receive-payments updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for receive-payments updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/receive-payments",
            page=SyncCursorPage[ReceivePayment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "customer_ids": customer_ids,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "limit": limit,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    receive_payment_list_params.ReceivePaymentListParams,
                ),
            ),
            model=ReceivePayment,
        )

    def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePaymentDeleteResponse:
        """Permanently deletes a a receive-payment.

        The deletion will fail if the
        receive-payment is currently in use or has any linked transactions that are in
        use.

        Args:
          id: The QuickBooks-assigned unique identifier of the receive-payment to delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._delete(
            f"/quickbooks-desktop/receive-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePaymentDeleteResponse,
        )


class AsyncReceivePaymentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReceivePaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReceivePaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReceivePaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncReceivePaymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        customer_id: str,
        total_amount: str,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        apply_to_transactions: Iterable[receive_payment_create_params.ApplyToTransaction] | NotGiven = NOT_GIVEN,
        credit_card_transaction: receive_payment_create_params.CreditCardTransaction | NotGiven = NOT_GIVEN,
        deposit_to_account_id: str | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        is_auto_apply: bool | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payment_method_id: str | NotGiven = NOT_GIVEN,
        receivables_account_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePayment:
        """
        Creates a new receive-payment.

        Args:
          customer_id: The customer or customer-job to which the payment for this receive-payment is
              credited.

          total_amount: The total monetary amount of this receive-payment, represented as a decimal
              string.

              **NOTE**: The sum of the `paymentAmount` amounts in the `applyToTransactions`
              array cannot exceed the `totalAmount`, or you will receive an error.

          transaction_date: The date of this receive-payment, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          apply_to_transactions: The invoices to be paid by this receive-payment. This will create a link between
              this receive-payment and the specified invoices.

              **IMPORTANT**: In each `applyToTransactions` object, you must specify either
              `paymentAmount`, `applyCredits`, `discountAmount`, or any combination of these;
              if none of these are specified, you will receive an error for an empty
              transaction.

              **IMPORTANT**: The target invoice must have `isPaid=false`, otherwise,
              QuickBooks will report this object as "cannot be found".

              **NOTE**: You must specify either `isAutoApply` or `applyToTransactions` when
              creating a receive-payment, but never both.

          credit_card_transaction: The credit card transaction data for this receive-payment's payment when using
              QuickBooks Merchant Services (QBMS). If specifying this field, you must also
              specify the `paymentMethod` field.

          deposit_to_account_id: The account where the funds for this receive-payment will be or have been
              deposited. If omitted, QuickBooks will use the default Undeposited Funds
              account.

          exchange_rate: The market exchange rate between this receive-payment's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          is_auto_apply: When `true`, QuickBooks applies `totalAmount` to any outstanding transaction
              that exactly matches `totalAmount`. If no exact match is found, this
              receive-payment is applied to the oldest outstanding transaction for the
              customer-job. When `false`, QuickBooks records the payment but does not apply it
              to any specific transaction, causing the amount to appear as a credit on the
              customer-job's next transaction.

              **IMPORTANT**: You must specify either `isAutoApply` or `applyToTransactions`
              when creating a receive-payment, but never both.

          memo: A memo or note for this receive-payment that will be displayed at the beginning
              of reports containing details about this receive-payment.

          payment_method_id: The receive-payment's payment method (e.g., cash, check, credit card).

              **NOTE**: If this receive-payment contains credit card transaction data supplied
              from QuickBooks Merchant Services (QBMS) transaction responses, you must specify
              a credit card payment method (e.g., "Visa", "MasterCard", etc.).

          receivables_account_id: The Accounts-Receivable (A/R) account to which this receive-payment is assigned,
              used to track the amount owed. If not specified, QuickBooks Desktop will use its
              default A/R account.

              **IMPORTANT**: If this receive-payment is linked to other transactions, this A/R
              account must match the `receivablesAccount` used in all linked transactions. For
              example, when refunding a credit card payment, the A/R account must match the
              one used in the original credit transactions being refunded.

          ref_number: The case-sensitive user-defined reference number for this receive-payment, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.
              When left blank in this create request, this field will be left blank in
              QuickBooks (i.e., it does _not_ auto-increment).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/receive-payments",
            body=await async_maybe_transform(
                {
                    "customer_id": customer_id,
                    "total_amount": total_amount,
                    "transaction_date": transaction_date,
                    "apply_to_transactions": apply_to_transactions,
                    "credit_card_transaction": credit_card_transaction,
                    "deposit_to_account_id": deposit_to_account_id,
                    "exchange_rate": exchange_rate,
                    "external_id": external_id,
                    "is_auto_apply": is_auto_apply,
                    "memo": memo,
                    "payment_method_id": payment_method_id,
                    "receivables_account_id": receivables_account_id,
                    "ref_number": ref_number,
                },
                receive_payment_create_params.ReceivePaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePayment,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePayment:
        """
        Retrieves a receive-payment by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the receive-payment to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/receive-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePayment,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        apply_to_transactions: Iterable[receive_payment_update_params.ApplyToTransaction] | NotGiven = NOT_GIVEN,
        credit_card_transaction: receive_payment_update_params.CreditCardTransaction | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        deposit_to_account_id: str | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payment_method_id: str | NotGiven = NOT_GIVEN,
        receivables_account_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        total_amount: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePayment:
        """
        Updates an existing receive-payment.

        Args:
          id: The QuickBooks-assigned unique identifier of the receive-payment to update.

          revision_number: The current QuickBooks-assigned revision number of the receive-payment object
              you are updating, which you can get by fetching the object first. Provide the
              most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          apply_to_transactions: The invoices to be paid by this receive-payment. This will create a link between
              this receive-payment and the specified invoices.

              **IMPORTANT**: In each `applyToTransactions` object, you must specify either
              `paymentAmount`, `applyCredits`, `discountAmount`, or any combination of these;
              if none of these are specified, you will receive an error for an empty
              transaction.

              **IMPORTANT**: The target invoice must have `isPaid=false`, otherwise,
              QuickBooks will report this object as "cannot be found".

          credit_card_transaction: The credit card transaction data for this receive-payment's payment when using
              QuickBooks Merchant Services (QBMS). If specifying this field, you must also
              specify the `paymentMethod` field.

          customer_id: The customer or customer-job to which the payment for this receive-payment is
              credited.

          deposit_to_account_id: The account where the funds for this receive-payment will be or have been
              deposited.

          exchange_rate: The market exchange rate between this receive-payment's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          memo: A memo or note for this receive-payment that will be displayed at the beginning
              of reports containing details about this receive-payment.

          payment_method_id: The receive-payment's payment method (e.g., cash, check, credit card).

          receivables_account_id: The Accounts-Receivable (A/R) account to which this receive-payment is assigned,
              used to track the amount owed. If not specified, QuickBooks Desktop will use its
              default A/R account.

              **IMPORTANT**: If this receive-payment is linked to other transactions, this A/R
              account must match the `receivablesAccount` used in all linked transactions. For
              example, when refunding a credit card payment, the A/R account must match the
              one used in the original credit transactions being refunded.

          ref_number: The case-sensitive user-defined reference number for this receive-payment, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          total_amount: The total monetary amount of this receive-payment, represented as a decimal
              string.

              **NOTE**: The sum of the `paymentAmount` amounts in the `applyToTransactions`
              array cannot exceed the `totalAmount`, or you will receive an error.

          transaction_date: The date of this receive-payment, in ISO 8601 format (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/receive-payments/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "apply_to_transactions": apply_to_transactions,
                    "credit_card_transaction": credit_card_transaction,
                    "customer_id": customer_id,
                    "deposit_to_account_id": deposit_to_account_id,
                    "exchange_rate": exchange_rate,
                    "memo": memo,
                    "payment_method_id": payment_method_id,
                    "receivables_account_id": receivables_account_id,
                    "ref_number": ref_number,
                    "total_amount": total_amount,
                    "transaction_date": transaction_date,
                },
                receive_payment_update_params.ReceivePaymentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePayment,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        customer_ids: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ReceivePayment, AsyncCursorPage[ReceivePayment]]:
        """Returns a list of receive-payments.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for receive-payments associated with these accounts.

          currency_ids: Filter for receive-payments in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          customer_ids: Filter for receive-payments received from these customers.

          ids: Filter for specific receive-payments by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          ref_number_contains: Filter for receive-payments whose `refNumber` contains this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with: Filter for receive-payments whose `refNumber` ends with this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for receive-payments whose `refNumber` is greater than or equal to this
              value. If omitted, the range will begin with the first number of the list. Uses
              a numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          ref_numbers: Filter for specific receive-payments by their ref-number(s), case-sensitive. In
              QuickBooks, ref-numbers are not required to be unique and can be arbitrarily
              changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ref_number_starts_with:
              Filter for receive-payments whose `refNumber` starts with this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberEndsWith`.

          ref_number_to: Filter for receive-payments whose `refNumber` is less than or equal to this
              value. If omitted, the range will end with the last number of the list. Uses a
              numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          transaction_date_from: Filter for receive-payments whose `date` field is on or after this date, in ISO
              8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for receive-payments whose `date` field is on or before this date, in ISO
              8601 format (YYYY-MM-DD).

          updated_after: Filter for receive-payments updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for receive-payments updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/receive-payments",
            page=AsyncCursorPage[ReceivePayment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "customer_ids": customer_ids,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "limit": limit,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    receive_payment_list_params.ReceivePaymentListParams,
                ),
            ),
            model=ReceivePayment,
        )

    async def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ReceivePaymentDeleteResponse:
        """Permanently deletes a a receive-payment.

        The deletion will fail if the
        receive-payment is currently in use or has any linked transactions that are in
        use.

        Args:
          id: The QuickBooks-assigned unique identifier of the receive-payment to delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._delete(
            f"/quickbooks-desktop/receive-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ReceivePaymentDeleteResponse,
        )


class ReceivePaymentsResourceWithRawResponse:
    def __init__(self, receive_payments: ReceivePaymentsResource) -> None:
        self._receive_payments = receive_payments

        self.create = to_raw_response_wrapper(
            receive_payments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            receive_payments.retrieve,
        )
        self.update = to_raw_response_wrapper(
            receive_payments.update,
        )
        self.list = to_raw_response_wrapper(
            receive_payments.list,
        )
        self.delete = to_raw_response_wrapper(
            receive_payments.delete,
        )


class AsyncReceivePaymentsResourceWithRawResponse:
    def __init__(self, receive_payments: AsyncReceivePaymentsResource) -> None:
        self._receive_payments = receive_payments

        self.create = async_to_raw_response_wrapper(
            receive_payments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            receive_payments.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            receive_payments.update,
        )
        self.list = async_to_raw_response_wrapper(
            receive_payments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            receive_payments.delete,
        )


class ReceivePaymentsResourceWithStreamingResponse:
    def __init__(self, receive_payments: ReceivePaymentsResource) -> None:
        self._receive_payments = receive_payments

        self.create = to_streamed_response_wrapper(
            receive_payments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            receive_payments.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            receive_payments.update,
        )
        self.list = to_streamed_response_wrapper(
            receive_payments.list,
        )
        self.delete = to_streamed_response_wrapper(
            receive_payments.delete,
        )


class AsyncReceivePaymentsResourceWithStreamingResponse:
    def __init__(self, receive_payments: AsyncReceivePaymentsResource) -> None:
        self._receive_payments = receive_payments

        self.create = async_to_streamed_response_wrapper(
            receive_payments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            receive_payments.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            receive_payments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            receive_payments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            receive_payments.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["SalesTaxCode", "SalesTaxItem"]


class SalesTaxItem(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class SalesTaxCode(BaseModel):
    id: str
    """The unique identifier assigned by QuickBooks to this sales-tax code.

    This ID is unique across all sales-tax codes but not across different QuickBooks
    object types.
    """

    created_at: str = FieldInfo(alias="createdAt")
    """
    The date and time when this sales-tax code was created, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """

    description: Optional[str] = None
    """A description of this sales-tax code."""

    is_active: bool = FieldInfo(alias="isActive")
    """Indicates whether this sales-tax code is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    is_taxable: bool = FieldInfo(alias="isTaxable")
    """Indicates whether this sales-tax code is tracking taxable sales.

    This field cannot be modified once the sales-tax code has been used in a
    transaction.
    """

    name: str
    """
    The case-insensitive unique name of this sales-tax code, unique across all
    sales-tax codes. This short name will appear on sales forms to identify the tax
    status of an item.

    **NOTE**: Sales-tax codes do not have a `fullName` field because they are not
    hierarchical objects, which is why `name` is unique for them but not for objects
    that have parents.
    """

    object_type: Literal["qbd_sales_tax_code"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_sales_tax_code"`."""

    revision_number: str = FieldInfo(alias="revisionNumber")
    """
    The current QuickBooks-assigned revision number of this sales-tax code object,
    which changes each time the object is modified. When updating this object, you
    must provide the most recent `revisionNumber` to ensure you're working with the
    latest data; otherwise, the update will return an error.
    """

    sales_tax_item: Optional[SalesTaxItem] = FieldInfo(alias="salesTaxItem", default=None)
    """
    The sales-tax item used to calculate the actual tax amount for this sales-tax
    code's transactions by applying a specific tax rate collected for a single tax
    agency. Unlike `salesTaxCode`, which only indicates general taxability, this
    field drives the actual tax calculation and reporting.
    """

    updated_at: str = FieldInfo(alias="updatedAt")
    """
    The date and time when this sales-tax code was last updated, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import standard_term_list_params, standard_term_create_params
from ..._base_client import make_request_options
from ...types.qbd.standard_term import StandardTerm
from ...types.qbd.standard_term_list_response import StandardTermListResponse

__all__ = ["StandardTermsResource", "AsyncStandardTermsResource"]


class StandardTermsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StandardTermsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return StandardTermsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StandardTermsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return StandardTermsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        discount_days: float | NotGiven = NOT_GIVEN,
        discount_percentage: str | NotGiven = NOT_GIVEN,
        due_days: float | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StandardTerm:
        """
        Creates a new standard term.

        Args:
          name: The case-insensitive unique name of this standard term, unique across all
              standard terms.

              **NOTE**: Standard terms do not have a `fullName` field because they are not
              hierarchical objects, which is why `name` is unique for them but not for objects
              that have parents.

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          discount_days: The number of days within which payment must be received to qualify for the
              discount specified by `discountPercentage`.

          discount_percentage: The discount percentage applied to the payment if received within the number of
              days specified by `discountDays`. The value is between 0 and 100.

          due_days: The number of days until payment is due.

          is_active: Indicates whether this standard term is active. Inactive objects are typically
              hidden from views and reports in QuickBooks. Defaults to `true`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/standard-terms",
            body=maybe_transform(
                {
                    "name": name,
                    "discount_days": discount_days,
                    "discount_percentage": discount_percentage,
                    "due_days": due_days,
                    "is_active": is_active,
                },
                standard_term_create_params.StandardTermCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StandardTerm,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StandardTerm:
        """
        Retrieves a standard term by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the standard term to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/standard-terms/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StandardTerm,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        names: List[str] | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StandardTermListResponse:
        """Returns a list of standard terms.

        NOTE: QuickBooks Desktop does not support
        pagination for standard terms; hence, there is no `cursor` parameter. Users
        typically have few standard terms.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          ids: Filter for specific standard terms by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return.

              **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
              standard terms. This parameter will limit the response size, but you cannot
              fetch subsequent results using a cursor. For pagination, use the name-range
              parameters instead (e.g., `nameFrom=A&nameTo=B`).

              When this parameter is omitted, the endpoint returns all standard terms without
              limit, unlike paginated endpoints which default to 150 records. This is
              acceptable because standard terms typically have low record counts.

          name_contains: Filter for standard terms whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for standard terms whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for standard terms whose `name` is alphabetically greater than or equal
              to this value.

          names: Filter for specific standard terms by their name(s), case-insensitive. Like
              `id`, `name` is a unique identifier for a standard term.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          name_starts_with: Filter for standard terms whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for standard terms whose `name` is alphabetically less than or equal to
              this value.

          status: Filter for standard terms that are active, inactive, or both.

          updated_after: Filter for standard terms updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for standard terms updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            "/quickbooks-desktop/standard-terms",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "names": names,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    standard_term_list_params.StandardTermListParams,
                ),
            ),
            cast_to=StandardTermListResponse,
        )


class AsyncStandardTermsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStandardTermsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStandardTermsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStandardTermsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncStandardTermsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        discount_days: float | NotGiven = NOT_GIVEN,
        discount_percentage: str | NotGiven = NOT_GIVEN,
        due_days: float | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StandardTerm:
        """
        Creates a new standard term.

        Args:
          name: The case-insensitive unique name of this standard term, unique across all
              standard terms.

              **NOTE**: Standard terms do not have a `fullName` field because they are not
              hierarchical objects, which is why `name` is unique for them but not for objects
              that have parents.

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          discount_days: The number of days within which payment must be received to qualify for the
              discount specified by `discountPercentage`.

          discount_percentage: The discount percentage applied to the payment if received within the number of
              days specified by `discountDays`. The value is between 0 and 100.

          due_days: The number of days until payment is due.

          is_active: Indicates whether this standard term is active. Inactive objects are typically
              hidden from views and reports in QuickBooks. Defaults to `true`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/standard-terms",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "discount_days": discount_days,
                    "discount_percentage": discount_percentage,
                    "due_days": due_days,
                    "is_active": is_active,
                },
                standard_term_create_params.StandardTermCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StandardTerm,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StandardTerm:
        """
        Retrieves a standard term by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the standard term to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/standard-terms/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StandardTerm,
        )

    async def list(
        self,
        *,
        conductor_end_user_id: str,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        names: List[str] | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> StandardTermListResponse:
        """Returns a list of standard terms.

        NOTE: QuickBooks Desktop does not support
        pagination for standard terms; hence, there is no `cursor` parameter. Users
        typically have few standard terms.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          ids: Filter for specific standard terms by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return.

              **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
              standard terms. This parameter will limit the response size, but you cannot
              fetch subsequent results using a cursor. For pagination, use the name-range
              parameters instead (e.g., `nameFrom=A&nameTo=B`).

              When this parameter is omitted, the endpoint returns all standard terms without
              limit, unlike paginated endpoints which default to 150 records. This is
              acceptable because standard terms typically have low record counts.

          name_contains: Filter for standard terms whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for standard terms whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for standard terms whose `name` is alphabetically greater than or equal
              to this value.

          names: Filter for specific standard terms by their name(s), case-insensitive. Like
              `id`, `name` is a unique identifier for a standard term.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          name_starts_with: Filter for standard terms whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for standard terms whose `name` is alphabetically less than or equal to
              this value.

          status: Filter for standard terms that are active, inactive, or both.

          updated_after: Filter for standard terms updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for standard terms updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            "/quickbooks-desktop/standard-terms",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "names": names,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    standard_term_list_params.StandardTermListParams,
                ),
            ),
            cast_to=StandardTermListResponse,
        )


class StandardTermsResourceWithRawResponse:
    def __init__(self, standard_terms: StandardTermsResource) -> None:
        self._standard_terms = standard_terms

        self.create = to_raw_response_wrapper(
            standard_terms.create,
        )
        self.retrieve = to_raw_response_wrapper(
            standard_terms.retrieve,
        )
        self.list = to_raw_response_wrapper(
            standard_terms.list,
        )


class AsyncStandardTermsResourceWithRawResponse:
    def __init__(self, standard_terms: AsyncStandardTermsResource) -> None:
        self._standard_terms = standard_terms

        self.create = async_to_raw_response_wrapper(
            standard_terms.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            standard_terms.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            standard_terms.list,
        )


class StandardTermsResourceWithStreamingResponse:
    def __init__(self, standard_terms: StandardTermsResource) -> None:
        self._standard_terms = standard_terms

        self.create = to_streamed_response_wrapper(
            standard_terms.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            standard_terms.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            standard_terms.list,
        )


class AsyncStandardTermsResourceWithStreamingResponse:
    def __init__(self, standard_terms: AsyncStandardTermsResource) -> None:
        self._standard_terms = standard_terms

        self.create = async_to_streamed_response_wrapper(
            standard_terms.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            standard_terms.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            standard_terms.list,
        )
